@@ -12,10 +12,7 @@ from ..Util import *
 
 if TYPE_CHECKING:
     from worlds._bizhawk.context import BizHawkClientContext
-    try:
-        from ..data.Entrances import PhantomHourglassEntrance
-    except ImportError:
-        print("Missing <game>Entrance dataclass")
+    from ..data.Entrances import PHTransition
 
 logger = logging.getLogger("Client")
 # Adding test edit
@@ -148,7 +145,7 @@ class DSZeldaClient(BizHawkClient):
         self.locations_in_scene = {}
         self.watches = {}
         self.receiving_location = False
-        self.last_vanilla_item: list[str] = []
+        self.last_vanilla_item: list[str | list[tuple[str, int]]] = []
         self.delay_reset = False
         self.getting_location = False
 
@@ -165,6 +162,7 @@ class DSZeldaClient(BizHawkClient):
         self.last_stage = None
         self.entering_from = None
         self.entering_dungeon = None
+        self.current_entrance = None
 
         self.stage_address = 0  # Used for scene flags
         self.new_stage_loading = None
@@ -178,8 +176,8 @@ class DSZeldaClient(BizHawkClient):
         self._log_received_items = False
 
         self.warp_to_start_flag = False
-        self.er_map: dict[int, dict["PhantomHourglassEntrance", "PhantomHourglassEntrance"]] = {}
-        self.er_in_scene: dict["PhantomHourglassEntrance", "PhantomHourglassEntrance"] | None = None
+        self.er_map: dict[int, dict["PHTransition", "PHTransition"]] = {}
+        self.er_in_scene: dict["PHTransition", "PHTransition"] | None = None
         self.er_exit_coord_writes: list | None = None
 
         self.delay_pickup = None
@@ -188,6 +186,8 @@ class DSZeldaClient(BizHawkClient):
         self.key_value = 0
         self.metal_count = 0
 
+        self.last_dungeon_warp_target = None
+
         self.tried_short_cs = False
 
     async def validate_rom(self, ctx: "BizHawkClientContext") -> bool:
@@ -302,6 +302,7 @@ class DSZeldaClient(BizHawkClient):
             self._loaded_menu_read_list = False
             self.last_scene = None
             self._from_menu = True
+            self.er_in_scene = None
             ctx.watcher_timeout = 0.5
             return
 
@@ -358,24 +359,27 @@ class DSZeldaClient(BizHawkClient):
                 await self.enter_game(ctx)
                 print(f"Started Game")
 
+            # getting_location can be overwritten in process_read_list
+            self.getting_location = read_result.get("getting_location", None)
+            self.is_dead = not read_result.get("link_health", 12)
+
             # Get current scene
             current_room = read_result.get("room", None)
-            current_room = 0 if current_room == 0xFF else current_room  # Resetting in a dungeon sets a special value
+            current_room = 0 if current_room == 0xFF and current_stage != 0x29 else current_room  # Resetting in a dungeon sets a special value
+            current_room = 3 if current_room == 0xFF else current_room
             self.current_scene = current_scene = current_stage * 0x100 + current_room
             current_entrance = read_result.get("entrance", 0)
             num_received_items = read_result.get("received_item_index", None)
 
-            # getting_location can be overwritten in process_read_list
-            self.getting_location = read_result.get("getting_location", None)
-            self.is_dead = not read_result.get("link_health", 12)
+
             await self.process_read_list(ctx, read_result)
 
             # Process on new room. As soon as it's triggered, changing the scene variable changes entrance destination
             if current_scene != self.last_scene and not self._entered_entrance and not self._loading_scene:
-
                 # Trigger a different entrance to vanilla
                 current_stage, current_room, current_entrance = await self._entrance_warp(ctx, current_scene, current_entrance)
                 current_scene = current_stage * 0x100 + current_room
+                self.current_entrance = current_entrance
 
                 # Backup in case of missing loading
                 self._backup_coord_read = await self.get_coords(ctx, multi=True)
@@ -396,7 +400,7 @@ class DSZeldaClient(BizHawkClient):
                     await self._remove_vanilla_item(ctx, num_received_items)
 
             # Nothing happens while loading
-            if not loading and not self._loading_scene and not self._entered_entrance:
+            if ctx.server is not None and not loading and not self._loading_scene and not self._entered_entrance:
 
                 # If new file, set up starting flags
                 if slot_memory == 0:
@@ -426,7 +430,7 @@ class DSZeldaClient(BizHawkClient):
                     await self._process_checked_locations(ctx, None, detection_type=self.getting_location_type)
 
                 # Process received items
-                if num_received_items < len(ctx.items_received):
+                if num_received_items is not None and num_received_items < len(ctx.items_received):
                     if self._just_entered_game:
                         self._log_received_items = True
                     await self._process_received_items(ctx, num_received_items, self._log_received_items)
@@ -453,23 +457,18 @@ class DSZeldaClient(BizHawkClient):
                         fallback, pickups = self.delay_pickup
                         need_fallback = True
                         for location, item, value in pickups:
-                            if "Small Key" in item:
-                                self.key_value = await read_memory_value(ctx, self.key_address)
-                                new_item_read = self.key_value
-                            else:
-                                check_item = ITEMS_DATA[item]
-                                new_item_read = await read_memory_value(ctx, check_item["address"],
-                                                                        check_item.get("size", 1))
+                            new_item_read = await self.get_item_read(ctx, item)
                             if "Rupee" in item:
                                 if new_item_read - value == ITEMS_DATA[item]["value"]:
-                                    await self._process_checked_locations(ctx, location, True)
+                                    await self._process_checked_locations(ctx, location, True, item=item)
                                     need_fallback = False
                             elif new_item_read != value:
-                                await self._process_checked_locations(ctx, location, True)
+                                await self._process_checked_locations(ctx, location, True, item=item)
                                 need_fallback = False
 
                         if need_fallback:
-                            await self._process_checked_locations(ctx, fallback, True)
+                            vanilla_item = LOCATIONS_DATA[fallback]["vanilla_item"]
+                            await self._process_checked_locations(ctx, fallback, True, item=vanilla_item)
 
                         self.delay_pickup = None
                         self.last_key_count = 0
@@ -617,8 +616,8 @@ class DSZeldaClient(BizHawkClient):
     async def _set_starting_flags(self, ctx: "BizHawkClientContext") -> None:
         write_list = [(self.slot_id_addr, split_bits(ctx.slot, 2), "Main RAM")]
         print(f"New game, setting starting flags for slot {ctx.slot}")
-        for adr, value in STARTING_FLAGS:
-            write_list.append((adr, [value], "Main RAM"))
+        for adr, *value in STARTING_FLAGS:
+            write_list.append((adr, value, "Main RAM"))
 
         write_list += await self.set_special_starting_flags(ctx)
         await bizhawk.write(ctx.bizhawk_ctx, write_list)
@@ -653,19 +652,30 @@ class DSZeldaClient(BizHawkClient):
                     (self.scene_addr[2], split_bits(0, 4), "Main RAM"),
                     (self.scene_addr[3], split_bits(e, 1), "Main RAM")]
 
-        def write_er(exit_d: "PhantomHourglassEntrance"):
-            write_res = write_entrance(*exit_d.entrance)
-            if exit_d.entrance[2] > 0xF0:
+        def write_er(exit_d: "PHTransition"):
+            if exit_d.entrance[2] == 0xFA:
+                # Special condition for exiting ships at sea
+                new_entrance = tuple(list(exit_d.entrance[:2]) + [exit_d.extra_data["ship_exit"]])
+                write_res = write_entrance(*new_entrance)
+            else:
+                write_res = write_entrance(*exit_d.entrance)
+
+            if exit_d.entrance[2] > 0xFA:
                 x, y, z = exit_d.coords
                 print(f"exit coords {x} {y} {z}")
                 self.er_exit_coord_writes = [(self.exit_coords_addr[0], split_bits(x, 4), "Main RAM"),
                                              (self.exit_coords_addr[1], split_bits(y, 4), "Main RAM"),
                                              (self.exit_coords_addr[2], split_bits(z, 4), "Main RAM")]
 
+            write_res += self.write_respawn_entrance(exit_d)
+
             return write_res
 
         def post_process(d):
             new_entrance = d.entrance
+            # Ship exits are weird
+            if new_entrance[2] == 0xFA:
+                new_entrance = tuple(list(new_entrance[:2]) + [d.extra_data["ship_exit"]])
             d.debug_print()
             return write_er(d), new_entrance
 
@@ -686,6 +696,7 @@ class DSZeldaClient(BizHawkClient):
             # Determine Entrance Warp
             coords = await self.get_coords(ctx)
             for detect_data, exit_data in self.er_in_scene.items():
+                print(f"trying to detect ER {res} {detect_data.entrance} {detect_data.detect_exit(going_to, entrance, coords, self.er_y_offest)}")
                 if detect_data.detect_exit(going_to, entrance, coords, self.er_y_offest):
                     if await self.conditional_er(ctx, exit_data):
                         print(f"Detected entrance: {detect_data} => {exit_data}")
@@ -704,12 +715,21 @@ class DSZeldaClient(BizHawkClient):
 
 
         if e_write_list:
+            print(f"Writing entrance warp {e_write_list}")
             await bizhawk.write(ctx.bizhawk_ctx, e_write_list)
         if defer_entrance:
             await self.store_visited_entrances(ctx, detect_data, exit_data)
 
         return res
 
+    def write_respawn_entrance(self, exit_data):
+        """
+        when at sea in ph with island shuffle on, the respawn point is not tied to the exit and must be set manually.
+        :param exit_data:
+        :return: list of write data
+        """
+        return []
+
     async def conditional_bounce(self, cxt, scene, entrance) -> "PhantomHourglassEntrance" or None:
         """
         checks for bounce conditions if entrance is not affected by ER.
@@ -794,7 +814,7 @@ class DSZeldaClient(BizHawkClient):
         # Write dynamic flags to memory
         read_list = {a: (a, 1, "Main RAM") for a in read_addr}
         prev = await read_memory_values(ctx, read_list)
-        print(f"{[[hex(int(a)), hex(v)] for a, v in prev.items()]}")
+        print(f"prevs: {[[hex(int(a)), hex(v)] for a, v in prev.items()]}")
 
         # Calculate values to write
         for a, v in set_bits.items():
@@ -804,7 +824,7 @@ class DSZeldaClient(BizHawkClient):
 
         # Write
         write_list = [(int(a), [v], "Main RAM") for a, v in prev.items()]
-        print(f"Dynaflags writes: {prev}")
+        print(f"Dynaflags writes: {[[hex(a), [hex(i) for i in v]] for a, v, _ in write_list]}")
         await bizhawk.write(ctx.bizhawk_ctx, write_list)
         return write_list
 
@@ -818,28 +838,49 @@ class DSZeldaClient(BizHawkClient):
 
             # Overwrite er_in_scene with dynamic entrance
             detect_data = data["detect_data"]
-            self.er_in_scene[detect_data] = data["exit_data"]
+            if data["exit_data"] is None:
+                if data["destination"] == "_connected_dungeon_entrance":
+                    self.er_in_scene[detect_data] = self.update_boss_warp(ctx, self.current_stage, scene)
+            else:
+                self.er_in_scene[detect_data] = data["exit_data"]
             print(f"\t{detect_data} => {data['exit_data']}")
 
     async def _has_dynamic_requirements(self, ctx, data) -> bool:
         def check_items(d):
             if "has_items" in d:
                 counter = [0] * len(d["has_items"])
-                for has_item in ctx.items_received:
-                    for i, want_item in enumerate(d["has_items"]):
-                        if has_item.item == ITEMS_DATA[want_item[0]]["id"]:
-                            counter[i] += 1
-                for item, count_have in zip(d["has_items"], counter):
-                    item, count_want, *operation = item
-                    if not operation:
-                        if (count_want == 0 and count_have != 0) or (count_want > 0 and count_have < count_want):
-                            return False
-                    elif operation[0] == "has_exact":
-                        if count_want != count_have:
-                            return False
-                    elif operation[0] == "not_has":
-                        if count_have >= count_want:
-                            return False
+                label = "has_items"
+            elif "not_has_all_items" in d:
+                counter = [0] * len(d["not_has_all_items"])
+                label = "not_has_all_items"
+            else:
+                return True
+
+            for has_item in ctx.items_received:
+                for i, want_item in enumerate(d[label]):
+                    if has_item.item == ITEMS_DATA[want_item[0]]["id"]:
+                        counter[i] += 1
+
+            for item, count_have in zip(d.get("has_items", []), counter):
+                item, count_want, *operation = item
+                if not operation:
+                    if (count_want == 0 and count_have != 0) or (count_want > 0 and count_have < count_want):
+                        return False
+                elif operation[0] == "has_exact":
+                    if count_want != count_have:
+                        return False
+                elif operation[0] == "not_has":
+                    if count_have >= count_want:
+                        return False
+
+            not_have_counter = 0
+            for item, count_have in zip(d.get("not_has_all_items", []), counter):
+                item, count_want, *operation = item
+                if count_have > count_want:
+                    not_have_counter += 1
+                if not_have_counter == len(counter):
+                    return False
+
             return True
 
         # Check location conditions
@@ -865,12 +906,17 @@ class DSZeldaClient(BizHawkClient):
         def check_slot_data(d):
             if "has_slot_data" in d:
                 for slot, value in d["has_slot_data"]:
-                    if ctx.slot_data.get(slot, None) != value:
-                        return False
+                    if type(value) is list:
+                        if ctx.slot_data.get(slot, None) not in value:
+                            return False
+                    else:
+                        if ctx.slot_data.get(slot, None) != value:
+                            return False
             return True
 
         # Came from particular location
         def check_last_room(d):
+            # print(f"checking last scene {self.last_scene} {d.get('last_scenes', [])}")
             for i in d.get("not_last_scenes", []):
                 if self.last_scene == i:
                     return False
@@ -882,13 +928,27 @@ class DSZeldaClient(BizHawkClient):
         # Read a dict of addresses to see if they match value
         async def check_bits(d):
             if "check_bits" in d:
-                r_list = {addr: (addr, 1, "Main RAM") for addr, _ in d["check_bits"].items()}
+                r_list = {addr: (addr, 1, "Main RAM") for addr, _, *args in d["check_bits"]}
+                v_lookup = {addr: v for addr, v, *args in d["check_bits"]}
+                arg_lookup = {addr: args for addr, v, *args in d["check_bits"] if args}
                 values = await read_memory_values(ctx, r_list)
                 for addr, p in values.items():
-                    if not (p & d["check_bits"][addr]):
-                        return False
+                    if not arg_lookup.get(addr, False):
+                        if not (p & v_lookup[addr]):
+                            return False
+                    elif "not" in arg_lookup[addr]:
+                        if p & v_lookup[addr]:
+                            return False
             return True
 
+        def has_entrance(d):
+            if "not_on_entrance" in d:
+                if self.current_entrance in d["not_on_entrance"]:
+                    return False
+            if "on_entrance" in d:
+                if self.current_entrance not in d["on_entrance"]:
+                    return False
+            return True
 
         if not check_items(data):
             print(f"\t{data['name']} does not have item reqs")
@@ -907,6 +967,8 @@ class DSZeldaClient(BizHawkClient):
             return False
         if not await self.has_special_dynamic_requirements(ctx, data):
             return False
+        if not has_entrance(data):
+            return False
 
         return True
 
@@ -921,7 +983,7 @@ class DSZeldaClient(BizHawkClient):
         return True
 
     async def _process_checked_locations(self, ctx: "BizHawkClientContext", pre_process: str = None, r=False,
-                                        detection_type=None):
+                                        detection_type=None, item: str | None = None):
         local_checked_locations = set()
         all_checked_locations = ctx.checked_locations
         location = None
@@ -932,7 +994,7 @@ class DSZeldaClient(BizHawkClient):
             loc_id = self.location_name_to_id[pre_process]
             location = LOCATIONS_DATA[pre_process]
             if r or (loc_id not in all_checked_locations):
-                await self._set_vanilla_item(ctx, location)
+                await self._set_vanilla_item(ctx, location, item)
                 local_checked_locations.add(loc_id)
             print(f"pre-processed {pre_process}, vanill {self.last_vanilla_item}")
         else:
@@ -968,18 +1030,20 @@ class DSZeldaClient(BizHawkClient):
                     local_checked_locations.add(loc_bytes)
                     await self._set_vanilla_item(ctx, location)
                     print(f"Got location {loc_name}! with vanilla {self.last_vanilla_item} id {loc_bytes}")
+                    self.locations_in_scene.pop(loc_name)  # Remove location for overlapping purposes
                     break
                 location = None
 
-        if location is not None and "set_bit" in location:
-            for addr, bit in location["set_bit"]:
-                print(f"Setting bit {bit} for location vanil {location['vanilla_item']}")
-                await write_memory_value(ctx, addr, bit)
+        if location is not None:
+            if "set_bit" in location:
+                for addr, bit in location["set_bit"]:
+                    print(f"Setting bit {bit} for location vanil {location['vanilla_item']}")
+                    await write_memory_value(ctx, addr, bit)
 
-        # Delay reset of vanilla item from certain address reads
-        if location is not None and "delay_reset" in location:
-            self.delay_reset = 1
-            print(f"Started Delay Reset for {self.last_vanilla_item}")
+            # Delay reset of vanilla item from certain address reads
+            if "delay_reset" in location:
+                self.delay_reset = 1
+                print(f"Started Delay Reset for {self.last_vanilla_item}")
 
         # Send locations
         # print(f"Local locations: {local_checked_locations} in \n{all_checked_locations}")
@@ -1002,6 +1066,13 @@ class DSZeldaClient(BizHawkClient):
         """
         return False
 
+    async def get_item_read(self, ctx, item_name: str) -> int:
+        if "Small Key" in item_name:
+            return await read_memory_value(ctx, self.key_address)
+        else:
+            item = ITEMS_DATA[item_name]
+            return await read_memory_value(ctx, item["address"], item.get("size", 1))
+
     async def _set_delay_pickup(self, ctx, loc_name, location):
         delay_locations = []
         delay_pickup = location["delay_pickup"]
@@ -1012,14 +1083,11 @@ class DSZeldaClient(BizHawkClient):
 
         self.delay_pickup = [loc_name, []]
         for loc in delay_locations:
-            delay_item_check = LOCATIONS_DATA[loc].get("vanilla_item", None)
-            if "Small Key" in delay_item_check:
-                self.last_key_count = await read_memory_value(ctx, self.key_address)
-                last_item_read = self.last_key_count
-            else:
-                last_item = ITEMS_DATA[delay_item_check]
-                last_item_read = await read_memory_value(ctx, last_item["address"], last_item.get("size", 1))
-            self.delay_pickup[1].append([loc, delay_item_check, last_item_read])
+            delay_item_check: str | list[str] = LOCATIONS_DATA[loc]["vanilla_item"]
+            if isinstance(delay_item_check, str):
+                delay_item_check = [delay_item_check]
+            for item in delay_item_check:
+                self.delay_pickup[1].append([loc, item, await self.get_item_read(ctx, item)])
         print(f"Delay pickup {self.delay_pickup}")
     # Processes events defined in data\dynamic_flags.py
 
@@ -1029,10 +1097,15 @@ class DSZeldaClient(BizHawkClient):
         print(f"Setting vanilla for {item_data}")
         if item is not None and not item_data.get("dummy", False):
             if ("incremental" in item_data or "progressive" in item_data or
-                    item_data["id"] not in [i.item for i in ctx.items_received]):
+                    item_data["id"] not in [i.item for i in ctx.items_received] or
+                    item_data.get("always_process", False)):
                 self.last_vanilla_item.append(item)
 
-                await self.unset_special_vanilla_items(ctx, location, item)
+                    await self.unset_special_vanilla_items(ctx, location, item)
+
+        # If there are multiple items possible at this location, store all of them with current counts for later
+        else:
+            self.last_vanilla_item.append([(_item, await self.get_item_read(ctx, _item)) for _item in item])
 
     async def unset_special_vanilla_items(self, ctx, location, item):
         """
@@ -1054,6 +1127,7 @@ class DSZeldaClient(BizHawkClient):
         :param location:
         :return:
         """
+        return
 
     async def _process_received_items(self, ctx: "BizHawkClientContext", num_received_items: int, log_items=False) -> None:
         # If the game hasn't received all items yet and the received item struct doesn't contain an item, then
@@ -1072,7 +1146,7 @@ class DSZeldaClient(BizHawkClient):
         print(f"Vanilla item: {self.last_vanilla_item} for {item_name}")
 
         # If same as vanilla item don't remove
-        if self.last_vanilla_item and item_name == self.last_vanilla_item[-1]:
+        if self.last_vanilla_item and item_name == self.last_vanilla_item[-1] and "Boss Key" not in item_name:
             self.last_vanilla_item.pop()
             print(f"oops it's vanilla or dummy! {self.last_vanilla_item}")
             write_list += await self.write_totok_keys_lol(ctx, item_name, item_data)
@@ -1246,39 +1320,53 @@ class DSZeldaClient(BizHawkClient):
     async def _remove_vanilla_item(self, ctx: "BizHawkClientContext", num_received_items):
         print(f"Removing vanilla items {self.last_vanilla_item}")
         for item in self.last_vanilla_item:
-            # Handle game specific items
-            if "dummy" in ITEMS_DATA[item]:
-                continue
-            if not await self.remove_special_vanilla_item(ctx, item):
-                data = ITEMS_DATA[item]
-                value = data.get('value', 1)
-                if "Small Key" in item:
-                    address = self.key_address = await self.get_small_key_address(ctx)
-                    print("small key?")
-                elif "progressive" in data:
-                    write_list = []
-                    index = sum([1 for i in ctx.items_received[:num_received_items] if i.item == data["id"]])
-                    if index >= len(data["progressive"]):
-                        continue
-                    address, value = data["progressive"][index]
-                    if "give_ammo" in data:
-                        ammo_v = data["give_ammo"][min(max(index - 1, 0), len(data["give_ammo"])-1)]
-                        write_list.append((data["ammo_address"], [ammo_v], "Main RAM"))
-                    # Progressive overwrite fix
-                    if "progressive_overwrite" in data and index > 1:
-                        write_list.append((data["progressive"][index-1][0], [data["progressive"][index-1][1]], "Main RAM"))
-                    await bizhawk.write(ctx.bizhawk_ctx, write_list)
-                else:
-                    address, value = data["address"], data.get("value", 1)
+            if isinstance(item, str):
+                # Handle game specific items
+                if "dummy" in ITEMS_DATA[item]:
+                    continue
+                if not await self.remove_special_vanilla_item(ctx, item):
+                    data = ITEMS_DATA[item]
+                    value = data.get('value', 1)
+                    if "Small Key" in item:
+                        address = self.key_address = await self.get_small_key_address(ctx)
+                        print("small key?")
+                    elif "progressive" in data:
+                        write_list = []
+                        index = sum([1 for i in ctx.items_received[:num_received_items] if i.item == data["id"]])
+                        if index >= len(data["progressive"]):
+                            continue
+                        address, value = data["progressive"][index]
+                        if "give_ammo" in data:
+                            ammo_v = data["give_ammo"][min(max(index - 1, 0), len(data["give_ammo"])-1)]
+                            write_list.append((data["ammo_address"], [ammo_v], "Main RAM"))
+                        # Progressive overwrite fix
+                        if "progressive_overwrite" in data and index > 1:
+                            write_list.append((data["progressive"][index-1][0], [data["progressive"][index-1][1]], "Main RAM"))
+                        await bizhawk.write(ctx.bizhawk_ctx, write_list)
+                    else:
+                        address, value = data["address"], data.get("value", 1)
 
-                # Catch vanilla rupees going over 9999
-                if "Rupee" in item:
-                    if self.prev_rupee_count + value > 9999:
-                        value =  9999 - self.prev_rupee_count
+                    # Catch vanilla rupees going over 9999
+                    if "Rupee" in item:
+                        if self.prev_rupee_count + value > 9999:
+                            value =  9999 - self.prev_rupee_count
 
-                await write_memory_value(ctx, address, value,
-                                         incr=data.get('incremental', None), unset=True, size=data.get("size", 1))
-        self.last_vanilla_item = []
+                    await write_memory_value(ctx, address, value,
+                                             incr=data.get('incremental', None), unset=True, size=data.get("size", 1))
+
+            # If item is a list of items, we instead want to check which one Link got and loop that back into this process
+            else:
+                for _item, _count in item:
+                    new_item_read = await self.get_item_read(ctx, _item)
+                    if "Rupee" in item:
+                        if new_item_read - _count == ITEMS_DATA[_item]["value"]:
+                            self.last_vanilla_item.append(_item)
+                            break
+                    elif new_item_read != _count:
+                        self.last_vanilla_item.append(_item)
+                        break
+
+        self.last_vanilla_item.clear()
     # Called during location processing to determine what vanilla item to remove
 
     async def remove_special_vanilla_item(self, ctx, vanilla_item: str):
@@ -1370,9 +1458,19 @@ class DSZeldaClient(BizHawkClient):
                 await self.update_key_count(ctx, stage)
         self.entering_from = scene_id
 
+    def update_boss_warp(self, ctx, stage, scene_id):
+        """
+        method for setting self.last_dungeon_warp_target for redirecting warps after bosses in entrance rando
+        :param ctx:
+        :param stage:
+        :param scene_id:
+        :return: PHTransition for the location
+        """
+        return None
+
     async def _load_local_locations(self, ctx, scene):
         # Load locations in room into loop
-        self.locations_in_scene = self.location_area_to_watches.get(scene, {})
+        self.locations_in_scene = self.location_area_to_watches.get(scene, {}).copy()
         print(f"Locations in scene {scene}: {self.locations_in_scene.keys()}")
         self.watches = {}
         sram_read_list = {}
@@ -1437,6 +1535,7 @@ class DSZeldaClient(BizHawkClient):
         if new_keys != 0:
             new_keys = 7 if new_keys >= 7 else new_keys
             new_keys, reset_key_count = await self.update_special_key_count(ctx, current_stage, new_keys, key_data, key_values, key_address)
+            new_keys = 0 if new_keys < 0 else new_keys
             write_list = [(key_address, [new_keys], "Main RAM")]
             if reset_key_count:
                 reset_tracker = (~key_data["filter"]) & key_values["tracker"]
@@ -1478,8 +1577,7 @@ class DSZeldaClient(BizHawkClient):
             if "locations" in hint_data:
                 # Hint required dungeons
                 if "Dungeon Hints" in hint_data["locations"]:
-                    for loc in ctx.slot_data.get("required_dungeon_locations", []):
-                        local_scouted_locations.add(self.location_name_to_id[loc])
+                    local_scouted_locations.update(self.dungeon_hints(ctx))
                 else:
                     locations_checked = ctx.locations_scouted
                     for loc in hint_data["locations"]:
@@ -1527,3 +1625,11 @@ class DSZeldaClient(BizHawkClient):
                      "room": room,
                      "entrance": entrance}
         }])
+
+    def dungeon_hints(self, ctx):
+        """
+        Write out dungeon hints depending on settings
+        :param ctx:
+        :return: list of location to scout
+        """
+        return []
