@@ -1,3 +1,5 @@
+
+
 VERSION = "0.3.0"
 ROM_HASH = "f2dc6c4e093e4f8c6cbea80e8dbd62cb"
 
@@ -206,9 +208,50 @@ SEA_REGIONS = [
     "Ocean Unspecific"
 ]
 
+SEA_CHARTS = [
+    "SW Sea Chart",
+    "NW Sea Chart",
+    "SE Sea Chart",
+    "NE Sea Chart",
+]
 
-ITEM_GROUPS = {
-    "Small Keys": [
+TREASURE_MAPS = [
+    "Treasure Map #1 (Molida SW)",
+    "Treasure Map #2 (Mercay NE)",
+    "Treasure Map #3 (Gusts SW)",
+    "Treasure Map #4 (Bannan SE)",
+    "Treasure Map #5 (Molida N)",
+    "Treasure Map #6 (Bannan W)",
+    "Treasure Map #7 (Gusts E)",
+    "Treasure Map #8 (Mercay SE)",
+    "Treasure Map #9 (Cannon W)",
+    "Treasure Map #10 (Gusts SE)",
+    "Treasure Map #11 (Gusts N)",
+    "Treasure Map #12 (Dee Ess N)",
+    "Treasure Map #13 (Harrow E)",
+    "Treasure Map #14 (Goron NW)",
+    "Treasure Map #15 (Goron W)",
+    "Treasure Map #16 (Goron NE)",
+    "Treasure Map #17 (Frost S)",
+    "Treasure Map #18 (Cannon S)",
+    "Treasure Map #19 (Gusts NE)",
+    "Treasure Map #20 (Bannan E)",
+    "Treasure Map #21 (Molida NW)",
+    "Treasure Map #22 (Harrow S)",
+    "Treasure Map #23 (Frost NW)",
+    "Treasure Map #24 (Ruins W)",
+    "Treasure Map #25 (Dead E)",
+    "Treasure Map #26 (Ruins SW)",
+    "Treasure Map #27 (Maze E)",
+    "Treasure Map #28 (Ruins NW)",
+    "Treasure Map #29 (Maze W)",
+    "Treasure Map #30 (Ruins S)",
+    "Treasure Map #31 (Dead S)",
+]
+
+
+ITEM_GROUPS: dict[str, set[str]] = {
+    "Small Keys": {
         "Small Key (Mountain Passage)",
         "Small Key (Temple of the Ocean King)",
         "Small Key (Temple of Fire)",
@@ -216,17 +259,13 @@ ITEM_GROUPS = {
         "Small Key (Temple of Courage)",
         "Small Key (Temple of Ice)",
         "Small Key (Mutoh's Temple)"
-    ],
-    "Vanilla Metals": [
+    },
+    "Vanilla Metals": {
         "Crimzonine",
         "Azurine",
         "Aquanine"
-    ],
-    "Metals": [
-        "Crimzonine",
-        "Azurine",
-        "Aquanine",
-        "Additional Rare Metal",
+    },
+    "Custom Metals": {
         "Verdanine",
         "Lavendine",
         "Amberine",
@@ -254,37 +293,8 @@ ITEM_GROUPS = {
         "Limeinine",
         "Mintine",
         "Umberine",
-    ],
-    "Custom Metals": [
-        "Verdanine",
-        "Lavendine",
-        "Amberine",
-        "Vermilline",
-        "Burgundine",
-        "Crystaline",
-        "Carrotine",
-        "Olivine",
-        "Chartreusine",
-        "Violetine",
-        "Ceruline",
-        "Fuchsianine",
-        "Saffronine",
-        "Viridine",
-        "Sepianine",
-        "Apricotine",
-        "Scarletine",
-        "Coraline",
-        "Magentine",
-        "Cyanine",
-        "Mauvine",
-        "Lilacine",
-        "Indigorine",
-        "Junipine",
-        "Limeinine",
-        "Mintine",
-        "Umberine",
-    ],
-    "Treasure Items": [
+    },
+    "Treasure Items": {
         "Treasure: Pink Coral",
         "Treasure: White Pearl Loop",
         "Treasure: Dark Pearl Loop",
@@ -293,97 +303,52 @@ ITEM_GROUPS = {
         "Treasure: Ruto Crown",
         "Treasure: Helmaroc Plume",
         "Treasure: Regal Ring"
-    ],
-    "Ammo Refills": [
+    },
+    "Ammo Refills": {
         "Refill: Bombs",
         "Refill: Arrows",
         "Refill: Bombchus",
         "Refill: Health",
-    ],
-    "Treasure Maps": [
-        "Treasure Map #1 (Molida SW)",
-        "Treasure Map #2 (Mercay NE)",
-        "Treasure Map #3 (Gusts SW)",
-        "Treasure Map #4 (Bannan SE)",
-        "Treasure Map #5 (Molida N)",
-        "Treasure Map #6 (Bannan W)",
-        "Treasure Map #7 (Gusts E)",
-        "Treasure Map #8 (Mercay SE)",
-        "Treasure Map #9 (Cannon W)",
-        "Treasure Map #10 (Gusts SE)",
-        "Treasure Map #11 (Gusts N)",
-        "Treasure Map #12 (Dee Ess N)",
-        "Treasure Map #13 (Harrow E)",
-        "Treasure Map #14 (Goron NW)",
-        "Treasure Map #15 (Goron W)",
-        "Treasure Map #16 (Goron NE)",
-        "Treasure Map #17 (Frost S)",
-        "Treasure Map #18 (Cannon S)",
-        "Treasure Map #19 (Gusts NE)",
-        "Treasure Map #20 (Bannan E)",
-        "Treasure Map #21 (Molida NW)",
-        "Treasure Map #22 (Harrow S)",
-        "Treasure Map #23 (Frost NW)",
-        "Treasure Map #24 (Ruins W)",
-        "Treasure Map #25 (Dead E)",
-        "Treasure Map #26 (Ruins SW)",
-        "Treasure Map #27 (Maze E)",
-        "Treasure Map #28 (Ruins NW)",
-        "Treasure Map #29 (Maze W)",
-        "Treasure Map #30 (Ruins S)",
-        "Treasure Map #31 (Dead S)",
-    ],
-    "Equipable Items": [
-        "Boomerang",
+    },
+    "Treasure Maps": set(TREASURE_MAPS),
+    "Items With Ammo": {
         "Bombs (Progressive)",
         "Bombchus (Progressive)",
         "Bow (Progressive)",
+    },
+    "Items Without Ammo": {
+        "Boomerang",
         "Grappling Hook",
         "Shovel",
         "Hammer",
-    ],
-    "Spirits": [
+    },
+    "Spirits": {
         "Spirit of Power (Progressive)",
         "Spirit of Wisdom (Progressive)",
         "Spirit of Courage (Progressive)",
-    ],
-    "Ship Items": [
+    },
+    "Ship Items": {
         "Cannon",
         "Salvage Arm",
         "Fishing Rod",
         "Big Catch Lure",
         "Cyclone Slate",
-    ],
-    "Fish": [
+    },
+    "Fish": {
         "Fish: Skippyjack",
         "Fish: Toona",
         "Fish: Loovar",
         "Fish: Rusty Swordfish",
         "Fish: Legendary Neptoona",
         "Fish: Stowfish"
-    ],
-    "Fishing Items": [
-        "Fishing Rod",
-        "Big Catch Lure",
-        "Fish: Skippyjack",
-        "Fish: Toona",
-        "Fish: Loovar",
-        "Fish: Rusty Swordfish",
-        "Fish: Legendary Neptoona",
-        "Fish: Stowfish"
-    ],
-    "Sea Charts": [
-        "SW Sea Chart",
-        "NW Sea Chart",
-        "SE Sea Chart",
-        "NE Sea Chart",
-    ],
-    "Upgrades": [
+    },
+    "Sea Charts": set(SEA_CHARTS),
+    "Upgrades": {
         "Heart Container",
         "Sand of Hours",
         "Swordsman's Scroll",
-    ],
-    "Rupees": [
+    },
+    "Rupees": {
         "Green Rupee (1)",
         "Blue Rupee (5)",
         "Red Rupee (20)",
@@ -393,104 +358,86 @@ ITEM_GROUPS = {
         "Rupoor (-10)",
         "Big Rupoor (-50)",
         "Pre-Alpha Rupee (5000)",
-    ],
-    "Throwable Keys": [
+    },
+    "Boss Keys": {
         "Boss Key (Temple of Fire)",
         "Boss Key (Temple of Wind)",
         "Boss Key (Temple of Courage)",
         "Boss Key (Goron Temple)",
         "Boss Key (Temple of Ice)",
         "Boss Key (Mutoh's Temple)",
-        "Square Crystal (Temple of Courage)",
-        "Triangle Crystal (Ghost Ship)",
-        "Round Crystal (Ghost Ship)",
-        "Round Crystal (Temple of the Ocean King)",
-        "Triangle Crystal (Temple of the Ocean King)",
-        "Force Gem (B3)",
-        "Force Gem (B12)",
-    ],
-    "Boss Keys": [
-        "Boss Key (Temple of Fire)",
-        "Boss Key (Temple of Wind)",
-        "Boss Key (Temple of Courage)",
-        "Boss Key (Goron Temple)",
-        "Boss Key (Temple of Ice)",
-        "Boss Key (Mutoh's Temple)",
-    ],
-    "Force Gems": [
-        "Force Gem (B3)",
-        "Force Gem (B12)",
-    ],
-    "Spirit Gems": [
+    },
+    "Single Spirit Gems": {
         "Power Gem",
         "Wisdom Gem",
         "Courage Gem",
-    ],
-    "Spirit Gem Packs": [
+    },
+    "Spirit Gem Packs": {
         "Power Gem Pack",
         "Wisdom Gem Pack",
         "Courage Gem Pack",
-    ],
-    "Shape Crystals": [
-        "Square Crystal (Temple of Courage)",
+    },
+
+    "Unique Crystal Items": {
+        "Triangle Pedestal B8 (Temple of the Ocean King)",
+        "Triangle Pedestal B9 (Temple of the Ocean King)",
+        "Round Pedestal B8 (Temple of the Ocean King)",
+        "Round Pedestal B9 (Temple of the Ocean King)",
+        "Square Pedestal West (Temple of the Ocean King)",
+        "Square Pedestal Center (Temple of the Ocean King)",
+        "Square Pedestal North (Temple of Courage)",
+        "Square Pedestal South (Temple of Courage)",
         "Triangle Crystal (Ghost Ship)",
         "Round Crystal (Ghost Ship)",
-        "Round Crystal (Temple of the Ocean King)",
-        "Triangle Crystal (Temple of the Ocean King)",
-    ],
-    "Collection Screen Keys": [
-        "Sun Key",
-        "Ghost Key",
-        "King's Key",
-        "Regal Necklace",
-    ],
-    "Keys": [
-        "Small Key (Mountain Passage)",
-        "Small Key (Temple of the Ocean King)",
-        "Small Key (Temple of Fire)",
-        "Small Key (Temple of Wind)",
-        "Small Key (Temple of Courage)",
-        "Small Key (Temple of Ice)",
-        "Small Key (Mutoh's Temple)",
-        "Boss Key (Temple of Fire)",
-        "Boss Key (Temple of Wind)",
-        "Boss Key (Temple of Courage)",
-        "Boss Key (Goron Temple)",
-        "Boss Key (Temple of Ice)",
-        "Boss Key (Mutoh's Temple)",
+    },
+    "Regular Crystal Items": {
         "Square Crystal (Temple of Courage)",
         "Triangle Crystal (Ghost Ship)",
         "Round Crystal (Ghost Ship)",
         "Round Crystal (Temple of the Ocean King)",
         "Triangle Crystal (Temple of the Ocean King)",
+        "Square Crystal (Temple of the Ocean King)",
+    },
+    "Global Crystal Items": {
+        "Square Crystals",
+        "Round Crystals",
+        "Triangle Crystals",
+    },
+    "Unique Force Gems": {
         "Force Gem (B3)",
         "Force Gem (B12)",
-        "Triforce Crest",
+    },
+    "Force Gems": {
+        "Force Gem (B3)",
+        "Force Gem (B12)",
+        "Force Gem"
+    },
+    "Collection Screen Keys": {
         "Sun Key",
         "Ghost Key",
         "King's Key",
         "Regal Necklace",
-    ],
-    "Trade Quest Items": [
+    },
+    "Trade Quest Items": {
         "Hero's New Clothes",
         "Kaleidoscope",
         "Guard Notebook",
         "Wood Heart",
-    ],
-    "Crests": [
+    },
+    "Crests": {
         "Courage Crest",
         "Triforce Crest",
-    ],
-    "Golden Frog Glyphs": [
+    },
+    "Golden Frog Glyphs": {
         "Golden Frog Glyph X",
         "Golden Frog Glyph Phi",
         "Golden Frog Glyph N",
         "Golden Frog Glyph Omega",
         "Golden Frog Glyph W",
         "Golden Frog Glyph Square"
-    ],
-    "Ships": [
-        "S.S. Linebeck",
+    },
+    "Ships": {
+#         "S.S. Linebeck",
         "Ship: Bright Ship",
         "Ship: Iron Ship",
         "Ship: Stone Ship",
@@ -499,38 +446,85 @@ ITEM_GROUPS = {
         "Ship: Tropical Ship",
         "Ship: Dignified Ship",
         "Ship: Golden Ship",
-    ],
-    "Technical Items": [
+    },
+    "Technical Items": {
         "Treasure",
         "Ship Part",
         "Potion",
         "Nothing!",
         "Sand of Hours (Boss)",
         "Sand of Hours (Small)",
-    ],
-    "Player Items": [
-        "Sword (Progressive)",
-        "Oshus' Sword",
-        "Phantom Sword",
-        "Shield",
-        "Boomerang",
-        "Bombs (Progressive)",
-        "Bombchus (Progressive)",
-        "Bow (Progressive)",
-        "Grappling Hook",
-        "Shovel",
-        "Hammer",
-    ],
-    "Rupoors": [
+        "Spirit of Courage (White)"
+    },
+    "Potions": {
+        "Red Potion",
+        "Purple Potion",
+        "Yellow Potion",
+        "Potion"
+    },
+    "Rupoors": {
         "Rupoor (-10)",
         "Big Rupoor (-50)",
-    ],
-    "Swords": [
+    },
+    "Swords": {
         "Sword (Progressive)",
         "Oshus' Sword",
         "Phantom Sword",
-    ]
+    },
+}
 
+# Combo groups
+ITEM_GROUPS |= {
+    "Metals":
+        ITEM_GROUPS["Vanilla Metals"] |
+        ITEM_GROUPS["Custom Metals"] |
+        {"Additional Rare Metal"},
+    "Equipable Items":
+        ITEM_GROUPS["Items With Ammo"] |
+        ITEM_GROUPS["Items Without Ammo"],
+    "Fishing Items": {
+                         "Fishing Rod",
+                         "Big Catch Lure"
+                     } |
+                     ITEM_GROUPS["Fish"],
+    "Regular Pedestal Items":
+        ITEM_GROUPS["Regular Crystal Items"] |
+        ITEM_GROUPS["Unique Force Gems"],
+    "Unique Pedestal Items":
+        ITEM_GROUPS["Unique Force Gems"] |
+        ITEM_GROUPS["Unique Crystal Items"],
+    "Global Pedestal Items":
+        ITEM_GROUPS["Global Crystal Items"] |
+        {"Force Gems"},
+    "Spirit Gems":
+        ITEM_GROUPS["Single Spirit Gems"] |
+        ITEM_GROUPS["Spirit Gem Packs"],
+}
+
+ITEM_GROUPS |=  {
+    "Shape Crystals":
+        ITEM_GROUPS["Unique Crystal Items"] |
+        ITEM_GROUPS["Regular Crystal Items"] |
+        ITEM_GROUPS["Global Crystal Items"],
+    "Pedestal Items":
+        ITEM_GROUPS["Regular Pedestal Items"] |
+        ITEM_GROUPS["Unique Pedestal Items"] |
+        ITEM_GROUPS["Global Pedestal Items"],
+    "Player Items": {
+                        "Shield",
+                    } | ITEM_GROUPS["Equipable Items"] |
+                    ITEM_GROUPS["Swords"],
+}
+
+ITEM_GROUPS |=  {
+    "Throwable Keys":
+        ITEM_GROUPS["Boss Keys"] |
+        ITEM_GROUPS["Pedestal Items"],
+    "Keys":
+        ITEM_GROUPS["Collection Screen Keys"] |
+        ITEM_GROUPS["Small Keys"] |
+        ITEM_GROUPS["Boss Keys"] |
+        ITEM_GROUPS["Pedestal Items"],
 }
 
 STAGE_LOCATION_GROUPS = {
@@ -805,8 +799,12 @@ STAGE_LOCATION_GROUPS = {
         "Temple of Fire 2F Rat Key",
         "Temple of Fire 3F Key Drop",
         "Temple of Fire 3F Boss Key Chest",
-        "Temple of Fire Blaaz Heart Container",
-        "Temple of Fire Blaaz Dungeon Reward",
+        "Blaaz Heart Container",
+        "Blaaz Boss Reward",
+    ],
+    "Blaaz": [
+        "Blaaz Heart Container",
+        "Blaaz Boss Reward",
     ],
     "Molida Island": [
         "Molida Island 2nd House Chest",
@@ -839,9 +837,14 @@ STAGE_LOCATION_GROUPS = {
         "Temple of Courage 1F Pols Voice Key 2",
         "Temple of Courage 2F Boss Key Chest",
         "Temple of Courage 3F Before Boss Chest",
-        "Temple of Courage Crayk Dungeon Reward",
-        "Temple of Courage Crayk Sand of Hours",
-        "Temple of Courage Heart Container",
+        "Crayk Boss Reward",
+        "Crayk Sand of Hours",
+        "Crayk Heart Container",
+    ],
+    "Crayk": [
+        "Crayk Boss Reward",
+        "Crayk Sand of Hours",
+        "Crayk Heart Container",
     ],
     "Spirit Island": [
         "Spirit Island Outside Chest",
@@ -874,9 +877,14 @@ STAGE_LOCATION_GROUPS = {
         "Temple of Wind B1 Key Drop",
         "Temple of Wind B2 Bomb Bag Chest",
         "Temple of Wind 1F Boss Key Chest",
-        "Temple of Wind Cyclok Sand of Hours",
-        "Temple of Wind Cyclok Dungeon Reward",
-        "Temple of Wind Cyclok Heart Container",
+        "Cyclok Sand of Hours",
+        "Cyclok Boss Reward",
+        "Cyclok Heart Container",
+    ],
+    "Cyclok": [
+        "Cyclok Sand of Hours",
+        "Cyclok Boss Reward",
+        "Cyclok Heart Container",
     ],
     "Bannan Island": [
         "Bannan Island Entrance Grapple Chest",
@@ -912,8 +920,12 @@ STAGE_LOCATION_GROUPS = {
         "Ghost Ship B2 Spike Chest",
         "Ghost Ship B3 Chest",
         "Ghost Ship Rescue Tetra",
-        "Ghost Ship Cubus Sisters Ghost Key",
-        "Ghost Ship Cubus Sisters Heart Container",
+        "Cubus Sisters Ghost Key",
+        "Cubus Sisters Heart Container",
+    ],
+    "Cubus Sisters": [
+        "Cubus Sisters Ghost Key",
+        "Cubus Sisters Heart Container",
     ],
     "Linebeck's Ship": [
         "Ocean Pirate Ambush Item",
@@ -935,9 +947,14 @@ STAGE_LOCATION_GROUPS = {
         "Goron Temple B3 Kill Miblins Chest",
         "Goron Temple B2 Kill Eyeslugs Chest",
         "Goron Temple B2 Boss Key Chest",
-        "Goron Temple Dongorongo Dungeon Reward",
-        "Goron Temple Dongorongo Sand of Hours",
-        "Goron Temple Dongorongo Heart Container",
+        "Dongorongo Boss Reward",
+        "Dongorongo Sand of Hours",
+        "Dongorongo Heart Container",
+    ],
+    "Dongorongo": [
+        "Dongorongo Boss Reward",
+        "Dongorongo Sand of Hours",
+        "Dongorongo Heart Container",
     ],
     "Harrow Island": [
         "Harrow Island Dig 1",
@@ -949,7 +966,7 @@ STAGE_LOCATION_GROUPS = {
         "Dee Ess Menu Button Dig",
         "Dee Ess Left Speakers Dig SSW",
         "Dee Ess Right Speakers Dig SE",
-        "Dee Ess Left Speakers Dig West ",
+        "Dee Ess Left Speakers Dig West",
         "Dee Ess Win Goron Game",
         "Dee Ess Eye Brute Chest",
         "Dee Ess Blow in Microphone Chest",
@@ -978,9 +995,14 @@ STAGE_LOCATION_GROUPS = {
         "Temple of Ice B2 Bow Bounce Chest",
         "Temple of Ice B2 Fight Chest",
         "Temple of Ice B2 Boss Key Chest",
-        "Temple of Ice Dungeon Reward",
-        "Temple of Ice Gleeok Sand of Hours",
-        "Temple of Ice Gleeok Heart Container",
+        "Gleeok Boss Reward",
+        "Gleeok Sand of Hours",
+        "Gleeok Heart Container",
+    ],
+    "Gleeok": [
+        "Gleeok Boss Reward",
+        "Gleeok Sand of Hours",
+        "Gleeok Heart Container",
     ],
     "Isle of the Dead": [
         "Isle of the Dead Rupoor Cave 1",
@@ -1010,10 +1032,14 @@ STAGE_LOCATION_GROUPS = {
         "Mutoh's Temple B1 Lower Water Chest",
         "Mutoh's Temple B1 Push Boulder Chest",
         "Mutoh's Temple B1 Boss Key Chest",
-        "Mutoh's Temple Dungeon Reward",
-        "Mutoh's Temple Eox Sand of Hours",
-        "Mutoh's Temple Heart Container Chest",
+        "Eox Boss Reward",
+        "Eox Sand of Hours",
+        "Eox Heart Container",
     ],
+    "Eox": [
+        "Eox Boss Reward",
+        "Eox Sand of Hours",
+        "Eox Heart Container",],
     "Maze Island": [
         "Maze Island Maze Chest",
         "Maze Island Beginner",
@@ -1090,8 +1116,8 @@ CATEGORY_LOCATION_GROUPS = {
         "Isle of Frost Estate SE Island Dig",
         "Isle of Frost Chief House Dig",
         "Isle of Frost Estate Sign Dig",
-        "Isle of Frost Fofo Dig (SE)",
-        "Isle of Frost Dobo Dig (SW)",
+        "Isle of Frost Estate Fofo Dig (SE)",
+        "Isle of Frost Estate Dobo Dig (SW)",
         "Dee Ess Left Speakers Dig SSW",
         "Dee Ess Right Speakers Dig SE",
         "Dee Ess Left Speakers Dig West ",
@@ -1190,6 +1216,14 @@ CATEGORY_LOCATION_GROUPS = {
         "Ocean NE Salvage #29 Maze W",
         "Ocean NE Salvage #30 Ruins S",
         "Ocean NE Salvage #31 Dead S",
+    ],
+    "Boss Rewards": [
+        "Blaaz Boss Reward",
+        "Cyclok Boss Reward",
+        "Crayk Boss Reward",
+        "Dongorongo Boss Reward",
+        "Gleeok Boss Reward",
+        "Eox Boss Reward",
     ]
 }
 
@@ -1221,18 +1255,19 @@ DUNGEON_NAMES = [
 
 DUNGEON_TO_BOSS_ITEM_LOCATION = {
     "Temple of the Ocean King": "TotOK B13 NE Sea Chart Chest",
-    "Temple of Fire": "Temple of Fire Blaaz Dungeon Reward",
-    "Temple of Wind": "Temple of Wind Cyclok Dungeon Reward",
-    "Temple of Courage": "Temple of Courage Crayk Dungeon Reward",
-    "Goron Temple": "Goron Temple Dongorongo Dungeon Reward",
-    "Temple of Ice": "Temple of Ice Dungeon Reward",
-    "Mutoh's Temple": "Mutoh's Temple Dungeon Reward",
+    "Temple of Fire": "Blaaz Boss Reward",
+    "Temple of Wind": "Cyclok Boss Reward",
+    "Temple of Courage": "Crayk Boss Reward",
+    "Goron Temple": "Dongorongo Boss Reward",
+    "Temple of Ice": "Gleeok Boss Reward",
+    "Mutoh's Temple": "Eox Boss Reward",
     "Ghost Ship": "_gs",
 }
 
 GHOST_SHIP_BOSS_ITEM_LOCATION = [
     "Ghost Ship Rescue Tetra",
-    "Ghost Ship Cubus Sisters Ghost Key"
+    "Cubus Sisters Ghost Key",
+    "Cubus Sisters Ghost Key"
 ]
 
 DUNGEON_KEY_DATA = {
@@ -1294,6 +1329,84 @@ DUNGEON_KEY_DATA = {
     },
 }
 
+BOSS_DOOR_DATA = {
+    0x1C: {
+        "name": "Temple of Fire",
+        "address": 0x258D20,
+        "value": 0xF
+    },
+    0x1D: {
+        "name": "Temple of Wind",
+        "address": 0x24D740,
+        "value": 0x77F
+    },
+    0x1E: {
+        "name": "Temple of Courage",
+        "address": 0x252360,
+        "value": 0x1FF
+    },
+    0x20: {
+        "name": "Goron Temple",
+        "address": 0x25D9B0,
+        "value": 0x1
+    },
+    0x1F: {
+        "name": "Temple of Ice",
+        "address": 0x259CA0,
+        "value": 0xDA
+    },
+    0x21: {
+        "name": "Mutoh's Temple",
+        "address": 0x24DED0,
+        "value": 0x1
+    },
+}
+
+# Entrance name: dungeon name
+BOSS_STAIRCASES = {
+    "ToF Enter Boss": "Temple of Fire",
+    "ToW Enter Boss": "Temple of Wind",
+    "ToC Enter Boss": "Temple of Courage",
+    "Ghost Ship Cubus Sisters Reunion": "Ghost Ship",
+    "GT Enter Boss": "Goron Temple",
+    "ToI Enter Boss": "Temple of Ice",
+    "MT Enter Boss": "Mutoh's Temple",
+}
+
+BOSS_LOOKUP = {
+    "Temple of Fire": "Blaaz",
+    "Temple of Wind": "Cyclok",
+    "Temple of Courage": "Crayk",
+    "Ghost Ship": "Cubus Sisters",
+    "Goron Temple": "Dongorongo",
+    "Temple of Ice": "Gleeok",
+    "Mutoh's Temple": "Eox"
+}
+
+# Boss Room Entrance name: boss reward location name
+BOSS_ENTRANCE_LOOKUP = {
+    "Blaaz Exit": "Blaaz Boss Reward",
+    "Cyclok Exit": "Cyclok Boss Reward",
+    "Crayk Exit": "Crayk Boss Reward",
+    "Cubus Sisters Blue Warp": "Cubus Sisters Ghost Key",
+    "Dongorongo Exit": "Dongorongo Boss Reward",
+    "Gleeok Exit": "Gleeok Boss Reward",
+    "Eox Exit": "Eox Boss Reward",
+}
+
+COLOR_SWITCH_DATA = {
+    0x1F: {
+        "name": "Temple of Ice",
+        "address": 0x20DBE0,
+        "value": 0x1
+    },
+    0x1E: {
+        "name": "Temple of Courage",
+        "address": 0x207CA8,
+        "value": 0x1
+    },
+}
+
 # Decode classification for humans
 CLASSIFICATION = {
     1: "progression",
@@ -1305,6 +1418,26 @@ CLASSIFICATION = {
     0: "filler"
                   }
 
+BOSS_WARP_LOOKUP = {
+    28: "ToF Exit",
+    29: "ToW Exit",
+    30: "ToC Exit",
+    31: "ToI Exit",
+    32: "GT Exit",
+    33: "MT Exit",
+    0x29: "Ghost Ship B1 Ascend"
+}
+
+BOSS_WARP_SCENE_LOOKUP = {
+    0x2B00: "Blaaz Exit",
+    0x2A00: "Cyclok Exit",
+    0x2C00: "Crayk Exit",
+    0x200A: "Dongo Exit",
+    0x1F06: "Gleeok Exit",
+    0x2106: "Eox Exit",
+    0x3000: "Cubus Sisters Blue Warp"
+}
+
 EQUIPPED_SHIP_PARTS_ADDR = [
     0x1BA544,
     0x1BA548,
